import { Rect } from "./dom";
declare global {
    interface Node {
        cmView: ContentView | undefined;
        cmIgnore: boolean | undefined;
    }
}
export declare const enum Dirty {
    Not = 0,
    Child = 1,
    Node = 2
}
export declare class DOMPos {
    readonly node: Node;
    readonly offset: number;
    readonly precise: boolean;
    constructor(node: Node, offset: number, precise?: boolean);
    static before(dom: Node, precise?: boolean): DOMPos;
    static after(dom: Node, precise?: boolean): DOMPos;
}
export declare abstract class ContentView {
    parent: ContentView | null;
    dom: Node | null;
    dirty: number;
    abstract length: number;
    abstract children: ContentView[];
    breakAfter: number;
    readonly overrideDOMText: ReadonlyArray<string> | null;
    readonly posAtStart: number;
    readonly posAtEnd: number;
    posBefore(view: ContentView): number;
    posAfter(view: ContentView): number;
    coordsAt(pos: number): Rect | null;
    sync(): void;
    reuseDOM(dom: Node): boolean;
    abstract domAtPos(pos: number): DOMPos;
    localPosFromDOM(node: Node, offset: number): number;
    domBoundsAround(from: number, to: number, offset?: number): {
        startDOM: Node | null;
        endDOM: Node | null;
        from: number;
        to: number;
    } | null;
    markDirty(andParent?: boolean): void;
    markParentsDirty(childList: boolean): void;
    setParent(parent: ContentView): void;
    setDOM(dom: Node): void;
    readonly rootView: ContentView;
    replaceChildren(from: number, to: number, children?: ContentView[]): void;
    ignoreMutation(rec: MutationRecord): boolean;
    ignoreEvent(event: Event): boolean;
    childCursor(pos?: number, i?: number): ChildCursor;
    childPos(pos: number, bias?: number): {
        i: number;
        off: number;
    };
    toString(): string;
}
export declare class ChildCursor {
    children: ReadonlyArray<ContentView>;
    pos: number;
    i: number;
    off: number;
    constructor(children: ReadonlyArray<ContentView>, pos: number, i: number);
    findPos(pos: number, bias?: number): this;
}
export declare class DocChildCursor extends ChildCursor {
    constructor(children: ReadonlyArray<ContentView>, pos: number, i: number);
    findPos(pos: number, bias?: number): this;
}
