import { EditorState, Transaction, ChangeSet } from "../../state";
import { StyleModule } from "style-mod";
import { Viewport } from "./viewport";
import { DecorationSet } from "./decoration";
import { Extension, Behavior, ExtensionGroup, Slot, SlotType } from "../../extension";
import { EditorView } from "./editorview";
import { Attrs } from "./attributes";
export declare type ViewCommand = (target: EditorView) => boolean;
export declare const extendView: ExtensionGroup<EditorView>;
export declare const handleDOMEvents: Behavior<{
    [key: string]: (view: EditorView, event: any) => boolean;
}, readonly {
    [key: string]: (view: EditorView, event: any) => boolean;
}[]>;
export declare const clickAddsSelectionRange: Behavior<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
export declare const dragMovesSelection: Behavior<(event: MouseEvent) => boolean, readonly ((event: MouseEvent) => boolean)[]>;
declare const pluginBehavior: unique symbol;
export declare class ViewPlugin<T extends ViewPluginValue<any>> {
    readonly create: (view: EditorView) => T;
    id: number;
    readonly extension: Extension;
    constructor(create: (view: EditorView) => T, behavior?: readonly {
        [pluginBehavior]: T;
    }[]);
    static behavior<Plugin, Input>(behavior: Behavior<Input, any>, read: (plugin: Plugin) => Input): {
        [pluginBehavior]: Plugin;
    };
    static decoration(spec: DecorationPluginSpec): Extension;
}
export interface DecorationPluginSpec {
    create: (view: EditorView) => DecorationSet;
    update: (deco: DecorationSet, update: ViewUpdate) => DecorationSet;
    map?: boolean;
}
export interface ViewPluginValue<Measure = undefined> {
    update?(update: ViewUpdate): void;
    draw?(): void;
    measure?(): Measure;
    drawMeasured?(measured: Measure): void;
    destroy?(): void;
}
export declare const editorAttributes: Behavior<Attrs, Attrs>;
export declare const contentAttributes: Behavior<Attrs, Attrs>;
export declare const viewPlugin: Behavior<ViewPlugin<any>, readonly ViewPlugin<any>[]>;
export declare const decorations: Behavior<import("../../rangeset/dist/rangeset").RangeSet<import("./decoration").Decoration>, readonly import("../../rangeset/dist/rangeset").RangeSet<import("./decoration").Decoration>[]>;
export declare const styleModule: Behavior<StyleModule<{}>, readonly StyleModule<{}>[]>;
export declare const theme: Behavior<StyleModule<{
    [key: string]: string;
}>, readonly StyleModule<{
    [key: string]: string;
}>[]>;
export declare const focusChange: SlotType<boolean>;
export declare const notified: SlotType<boolean>;
export declare class ViewUpdate {
    readonly view: EditorView;
    readonly transactions: ReadonlyArray<Transaction>;
    readonly metadata: ReadonlyArray<Slot>;
    readonly state: EditorState;
    readonly changes: ChangeSet;
    readonly prevState: EditorState;
    readonly prevViewport: Viewport;
    private prevThemes;
    constructor(view: EditorView, transactions?: ReadonlyArray<Transaction>, metadata?: ReadonlyArray<Slot>);
    readonly viewport: {
        from: number;
        to: number;
    };
    readonly viewportChanged: boolean;
    readonly docChanged: boolean;
    readonly themeChanged: boolean;
    getMeta<T>(type: SlotType<T>): T | undefined;
}
export {};
