import { ContentView, ChildCursor, DOMPos } from "./contentview";
import { BlockView } from "./blockview";
import { Viewport, ViewportState } from "./viewport";
import { DOMObserver } from "./domobserver";
import { HeightMap, HeightOracle, BlockInfo } from "./heightmap";
import { DecorationSet } from "./decoration";
import { Rect } from "./dom";
import { ViewUpdate } from "./extension";
import { EditorView } from "./editorview";
import { EditorState, ChangedRange } from "../../state";
declare type A<T> = ReadonlyArray<T>;
export declare class DocView extends ContentView {
    private view;
    children: BlockView[];
    viewports: Viewport[];
    decorations: A<DecorationSet>;
    compositionDeco: DecorationSet;
    gapDeco: DecorationSet;
    selectionDirty: any;
    observer: DOMObserver;
    forceSelectionUpdate: boolean;
    viewportState: ViewportState;
    heightMap: HeightMap;
    heightOracle: HeightOracle;
    layoutCheckScheduled: number;
    scrollIntoView: number;
    paddingTop: number;
    paddingBottom: number;
    minWidth: number;
    minWidthFrom: number;
    minWidthTo: number;
    impreciseAnchor: DOMPos | null;
    impreciseHead: DOMPos | null;
    dom: HTMLElement;
    readonly length: number;
    readonly state: EditorState;
    readonly viewport: Viewport;
    readonly root: DocumentOrShadowRoot;
    constructor(view: EditorView, onDOMChange: (from: number, to: number, typeOver: boolean) => boolean);
    init(state: EditorState, initialize: (viewport: Viewport) => void): void;
    update(update: ViewUpdate | null, scrollIntoView?: number): void;
    private updateInner;
    private updateChildren;
    private replaceRange;
    updateSelection(takeFocus?: boolean): void;
    lineAt(pos: number, editorTop?: number): BlockInfo;
    lineAtHeight(height: number, editorTop?: number): BlockInfo;
    blockAtHeight(height: number, editorTop?: number): BlockInfo;
    forEachLine(from: number, to: number, f: (line: BlockInfo) => void, editorTop?: number): any;
    computeUpdate(state: EditorState, update: ViewUpdate | null, initializing: null | ((viewport: Viewport) => void), contentChanges: A<ChangedRange>, viewportBias: number, scrollIntoView: number): A<ChangedRange>;
    focus(): void;
    cancelLayoutCheck(): void;
    checkLayout(forceFull?: boolean): void;
    scrollPosIntoView(pos: number): void;
    nearest(dom: Node): ContentView | null;
    posFromDOM(node: Node, offset: number): number;
    domAtPos(pos: number): DOMPos;
    coordsAt(pos: number): Rect | null;
    measureVisibleLineHeights(): number[];
    measureVerticalPadding(): void;
    measureTextSize(): {
        lineHeight: number;
        charWidth: number;
    };
    destroy(): void;
    clearSelectionDirty(): void;
    setSelectionDirty(): void;
    childCursor(pos?: number, i?: number): ChildCursor;
    computeGapDeco(viewports: A<Viewport>, docLength: number): DecorationSet;
}
export declare function computeCompositionDeco(view: EditorView, changes: A<ChangedRange>): DecorationSet;
export {};
